==> cloud-4.1.0.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/src/Form/Ec2/VolumeCreateForm.php <==
<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\aws_cloud\Traits\AwsCloudFormTrait;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the Volume entity create form.
 *
 * @ingroup aws_cloud
 */
class VolumeCreateForm extends AwsCloudContentForm {

  use AwsCloudFormTrait;
  use CloudContentEntityTrait;

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = ''): array {

    $this->ec2Service->setCloudContext($cloud_context);

    /** @var \Drupal\aws_cloud\Entity\Ec2\Volume $entity */
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;

    // Get module name.
    $module_name = $this->getModuleName($entity);

    // Use the value of parameter snapshot_id as the default value.
    $snapshot_id = $this->getRequest()->query->get('snapshot_id');
    $snapshot = NULL;
    if (!empty($snapshot_id)) {
      $snapshots = $this->entityTypeManager
        ->getStorage("{$module_name}_snapshot")
        ->loadByProperties([
          'cloud_context' => $cloud_context,
          'snapshot_id' => $snapshot_id,
        ]);

      if (!empty($snapshots)) {
        $snapshot = reset($snapshots);
      }
    }

    $weight = -50;

    $form['volume'] = [
      '#type' => 'details',
      '#title' => $this->t('@title', ['@title' => $entity->getEntityType()->getSingularLabel()]),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['volume']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->label(),
      '#weight'        => -5,
      '#required'      => TRUE,
    ];

    $form['volume']['snapshot_id'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Snapshot ID'),
      '#options'       => $this->getSnapshotOptions($cloud_context, $module_name),
      '#default_value' => $snapshot_id,
      '#weight'        => -5,
      '#required'      => FALSE,
      '#empty_value'   => '',
    ];

    $form['volume']['size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Size (GiB)'),
      '#size'          => 60,
      '#default_value' => $snapshot ? $snapshot->getSize() : '',
      '#required'      => TRUE,
    ];

    $form['volume']['volume_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Volume type'),
      '#options' => [
        'gp2' => $this->t('General Purpose SSD (gp2)'),
        'gp3' => $this->t('General Purpose SSD (gp3)'),
        'io1' => $this->t('Provisioned IOPS SSD (io1)'),
        'io2' => $this->t('Provisioned IOPS SSD (io2)'),
        'sc1' => $this->t('Cold HDD (sc1)'),
        'st1' => $this->t('Throughput Optimized HDD (st1)'),
        'standard' => $this->t('Magnetic (standard)'),
      ],
    ];

    $form['volume']['iops'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('IOPS'),
      '#size'          => 60,
      '#default_value' => $entity->getIops(),
      '#required'      => FALSE,
    ];

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $availability_zone_options = $this->getAvailabilityZoneOptions($cloud_context, $module_name);

    $form['volume']['availability_zone'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Availability Zone'),
      '#options'       => $availability_zone_options,
      // Pick up the first availability zone in the array.
      '#default_value' => array_shift($availability_zone_options),
      '#required'      => TRUE,
    ];

    $form['volume']['kms_key_id'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('KMS key ID'),
      '#size'          => 60,
      '#default_value' => $entity->getKmsKeyId(),
      '#required'      => FALSE,
    ];

    $form['volume']['encrypted'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Encrypted'),
      '#size'          => 60,
      '#default_value' => $entity->getEncrypted(),
      '#required'      => FALSE,

    ];

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);

    // Add validation for volume types.  for io1 type,
    // iops must be set.  For other volume types, iops cannot
    // be set.
    if ($form_state->getValue('volume_type') === 'io1') {
      // Check if there is an iops value.
      if (empty($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('Specify an iops value.  The value must be a minimum of 100.'));
      }

      // Check if iops is an integer.
      if (!is_numeric($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('IOPS must be an integer.'));
      }
      // Check if iops is greater than 100.
      $iops = (int) $form_state->getValue('iops');
      if ($iops < 100) {
        $form_state->setErrorByName('iops', $this->t('IOPS must be a minimum of 100.'));
      }
    }
    else {
      if (!empty($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('IOPS cannot be set unless volume type is "Provisioned IOPS SSD".'));
      }
    }
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\aws_cloud\Service\Ec2\Ec2ServiceException
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $this->trimTextfields($form, $form_state);

    $entity = $this->entity;

    $params = [
      'Name'             => $entity->getName(),
      'SnapshotId'       => $entity->getSnapshotId(),
      'Size'             => $entity->getSize(),
      'AvailabilityZone' => $entity->getAvailabilityZone(),
      'VolumeType'       => $entity->getVolumeType(),
      'Encrypted'        => $entity->getEncrypted() ? TRUE : FALSE,
    ];

    if ($entity->getVolumeType() === 'io1') {
      $params['Iops'] = (int) $entity->getIops();
    }

    if (!empty($entity->getKmsKeyId())) {
      $params['KmsKeyId'] = $entity->getKmsKeyId();
    }

    if (!empty($entity->getSnapshotId())) {
      $params['SnapshotId'] = $entity->getSnapshotId();
    }

    $form_state->setRedirect("view.{$entity->getEntityTypeId()}.list", ['cloud_context' => $entity->getCloudContext()]);

    $result = $this->ec2Service->createVolume($params);
    if ($result === NULL) {
      $this->processOperationErrorStatus($entity, 'created');
      return;
    }

    if (!empty($result['SendToWorker'])) {
      $this->processOperationStatus($entity, 'created remotely');
      return;
    }

    if (!($entity->setVolumeId($result['VolumeId'])
      && $entity->setCreated(strtotime($result['CreateTime']))
      && $entity->setState($result['State'])
      && $entity->setSnapshotName($this->getSnapshotName($entity->getCloudContext(), $entity->getSnapshotId(), $this->getModuleName($entity)))
      && $entity->setVolumeType($result['VolumeType'])
      && $entity->save())) {

      $this->processOperationErrorStatus($entity, 'created');
      return;
    }

    $this->updateNameAndCreatedByTags($entity, $entity->getVolumeId());

    $this->processOperationStatus($entity, 'created');
    $this->clearCacheValues($entity->getCacheTags());
    $this->dispatchSaveEvent($entity);
  }

  /**
   * Get Snapshot Name.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param string $snapshot_id
   *   Snapshot ID.
   * @param string $module_name
   *   Module name.
   *
   * @return string
   *   Snapshot Name.
   */
  private function getSnapshotName($cloud_context, $snapshot_id, $module_name): string {
    $snapshot_name = '';
    if (empty($snapshot_id)) {
      return $snapshot_name;
    }

    $snapshots = $this->entityTypeManager
      ->getStorage("{$module_name}_snapshot")
      ->loadByProperties([
        'cloud_context' => $cloud_context,
        'snapshot_id' => $snapshot_id,
      ]);

    if (empty($snapshots)) {
      return $snapshot_name;
    }

    return array_shift($snapshots)->getName();
  }

  /**
   * Helper function to get snapshot options.
   *
   * @param string $cloud_context
   *   Cloud context to use in the query.
   * @param string $module_name
   *   Module name.
   *
   * @return array
   *   Snapshot options.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getSnapshotOptions($cloud_context, $module_name): array {
    $options = [];
    $params = [
      'cloud_context' => $cloud_context,
    ];

    // Get cloud service provider name.
    $cloud_name = self::convertUnderscoreToWhitespace($module_name);
    if (!$this->currentUser->hasPermission("view any {$cloud_name} snapshot")) {
      $params['uid'] = $this->currentUser->id();
    }

    $snapshots = $this->entityTypeManager
      ->getStorage("{$module_name}_snapshot")
      ->loadByProperties($params);
    foreach ($snapshots ?: [] as $snapshot) {
      if ($snapshot->getName() !== $snapshot->getSnapshotId()) {
        $options[$snapshot->getSnapshotId()] = "{$snapshot->getName()} ({$snapshot->getSnapshotId()})";
      }
      else {
        $options[$snapshot->getSnapshotId()] = $snapshot->getSnapshotId();
      }
    }

    natcasesort($options);
    return $options;
  }

}

==> cloud-4.3.0.tar.gz: cloud/modules/cloud_service_providers/aws_cloud/src/Form/Ec2/VolumeCreateForm.php <==
<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\aws_cloud\Traits\AwsCloudFormTrait;
use Drupal\cloud\Service\CloudService;
use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form controller for the Volume entity create form.
 *
 * @ingroup aws_cloud
 */
class VolumeCreateForm extends AwsCloudContentForm {

  use AwsCloudFormTrait;
  use CloudContentEntityTrait;

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   * @param string $cloud_context
   *   A cloud_context string value from URL "path".
   *
   * @return array
   *   Array of form object.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = ''): array {

    $this->ec2Service->setCloudContext($cloud_context);

    /** @var \Drupal\aws_cloud\Entity\Ec2\Volume $entity */
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;

    // Get module name.
    $module_name = $this->getModuleName($entity);

    // Use the value of parameter snapshot_id as the default value.
    $snapshot_id = $this->getRequest()->query->get('snapshot_id');
    $snapshot = NULL;
    if (!empty($snapshot_id)) {
      $snapshots = $this->entityTypeManager
        ->getStorage("{$module_name}_snapshot")
        ->loadByProperties([
          'cloud_context' => $cloud_context,
          'snapshot_id' => $snapshot_id,
        ]);

      if (!empty($snapshots)) {
        $snapshot = reset($snapshots);
      }
    }

    $weight = -50;

    $form['volume'] = [
      '#type' => 'details',
      '#title' => $this->t('@title', ['@title' => $entity->getEntityType()->getSingularLabel()]),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['volume']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->label(),
      '#weight'        => -5,
      '#required'      => TRUE,
    ];

    $form['volume']['snapshot_id'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Snapshot ID'),
      '#options'       => $this->getSnapshotOptions($cloud_context, $module_name),
      '#default_value' => $snapshot_id,
      '#weight'        => -5,
      '#required'      => FALSE,
      '#empty_value'   => '',
    ];

    $form['volume']['size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Size (GiB)'),
      '#size'          => 60,
      '#default_value' => $snapshot ? $snapshot->getSize() : '',
      '#required'      => TRUE,
    ];

    $form['volume']['volume_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Volume type'),
      '#options' => [
        'gp2' => $this->t('General Purpose SSD (gp2)'),
        'gp3' => $this->t('General Purpose SSD (gp3)'),
        'io1' => $this->t('Provisioned IOPS SSD (io1)'),
        'io2' => $this->t('Provisioned IOPS SSD (io2)'),
        'sc1' => $this->t('Cold HDD (sc1)'),
        'st1' => $this->t('Throughput Optimized HDD (st1)'),
        'standard' => $this->t('Magnetic (standard)'),
      ],
    ];

    $form['volume']['iops'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('IOPS'),
      '#size'          => 60,
      '#default_value' => $entity->getIops(),
      '#required'      => FALSE,
    ];

    // Get module name.
    $module_name = $this->getModuleName($entity);

    $availability_zone_options = $this->getAvailabilityZoneOptions($cloud_context, $module_name);

    $form['volume']['availability_zone'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Availability Zone'),
      '#options'       => $availability_zone_options,
      // Pick up the first availability zone in the array.
      '#default_value' => array_shift($availability_zone_options),
      '#required'      => TRUE,
    ];

    $form['volume']['kms_key_id'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('KMS key ID'),
      '#size'          => 60,
      '#default_value' => $entity->getKmsKeyId(),
      '#required'      => FALSE,
    ];

    $form['volume']['encrypted'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Encrypted'),
      '#size'          => 60,
      '#default_value' => $entity->getEncrypted(),
      '#required'      => FALSE,

    ];

    $this->addOthersFieldset($form, $weight++, $cloud_context);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);

    // Add validation for volume types.  for io1 type,
    // iops must be set.  For other volume types, iops cannot
    // be set.
    if ($form_state->getValue('volume_type') === 'io1') {
      // Check if there is an iops value.
      if (empty($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('Specify an iops value.  The value must be a minimum of 100.'));
      }

      // Check if iops is an integer.
      if (!is_numeric($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('IOPS must be an integer.'));
      }
      // Check if iops is greater than 100.
      $iops = (int) $form_state->getValue('iops');
      if ($iops < 100) {
        $form_state->setErrorByName('iops', $this->t('IOPS must be a minimum of 100.'));
      }
    }
    else {
      if (!empty($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('IOPS cannot be set unless volume type is "Provisioned IOPS SSD".'));
      }
    }
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   *
   * @param array $form
   *   Array of form object.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\aws_cloud\Service\Ec2\Ec2ServiceException
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $this->trimTextfields($form, $form_state);

    $entity = $this->entity;

    $params = [
      'Name'             => $entity->getName(),
      'SnapshotId'       => $entity->getSnapshotId(),
      'Size'             => $entity->getSize(),
      'AvailabilityZone' => $entity->getAvailabilityZone(),
      'VolumeType'       => $entity->getVolumeType(),
      'Encrypted'        => $entity->getEncrypted() ? TRUE : FALSE,
    ];

    if ($entity->getVolumeType() === 'io1') {
      $params['Iops'] = (int) $entity->getIops();
    }

    if (!empty($entity->getKmsKeyId())) {
      $params['KmsKeyId'] = $entity->getKmsKeyId();
    }

    if (!empty($entity->getSnapshotId())) {
      $params['SnapshotId'] = $entity->getSnapshotId();
    }

    $form_state->setRedirect("view.{$entity->getEntityTypeId()}.list", ['cloud_context' => $entity->getCloudContext()]);

    $result = $this->ec2Service->createVolume($params);
    if ($result === NULL) {
      $this->processOperationErrorStatus($entity, 'created');
      return;
    }

    if (!empty($result['SendToWorker'])) {
      $this->processOperationStatus($entity, 'created remotely');
      return;
    }

    if (!($entity->setVolumeId($result['VolumeId'])
      && $entity->setCreated(strtotime($result['CreateTime']))
      && $entity->setState($result['State'])
      && $entity->setSnapshotName($this->getSnapshotName($entity->getCloudContext(), $entity->getSnapshotId(), $this->getModuleName($entity)))
      && $entity->setVolumeType($result['VolumeType'])
      && $entity->save())) {

      $this->processOperationErrorStatus($entity, 'created');
      return;
    }

    $this->updateNameAndCreatedByTags($entity, $entity->getVolumeId());

    $this->processOperationStatus($entity, 'created');
    $this->clearCacheValues($entity->getCacheTags());
    $this->dispatchSaveEvent($entity);
  }

  /**
   * Get Snapshot Name.
   *
   * @param string $cloud_context
   *   The cloud context.
   * @param string $snapshot_id
   *   Snapshot ID.
   * @param string $module_name
   *   Module name.
   *
   * @return string
   *   Snapshot Name.
   */
  private function getSnapshotName($cloud_context, $snapshot_id, $module_name): string {
    $snapshot_name = '';
    if (empty($snapshot_id)) {
      return $snapshot_name;
    }

    $snapshots = $this->entityTypeManager
      ->getStorage("{$module_name}_snapshot")
      ->loadByProperties([
        'cloud_context' => $cloud_context,
        'snapshot_id' => $snapshot_id,
      ]);

    if (empty($snapshots)) {
      return $snapshot_name;
    }

    return array_shift($snapshots)->getName();
  }

  /**
   * Helper function to get snapshot options.
   *
   * @param string $cloud_context
   *   Cloud context to use in the query.
   * @param string $module_name
   *   Module name.
   *
   * @return array
   *   Snapshot options.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getSnapshotOptions($cloud_context, $module_name): array {
    $options = [];
    $params = [
      'cloud_context' => $cloud_context,
    ];

    // Get cloud service provider name.
    // Use a static trait method through CloudService.
    $cloud_name = CloudService::convertUnderscoreToWhitespace($module_name);
    if (!$this->currentUser->hasPermission("view any {$cloud_name} snapshot")) {
      $params['uid'] = $this->currentUser->id();
    }

    $snapshots = $this->entityTypeManager
      ->getStorage("{$module_name}_snapshot")
      ->loadByProperties($params);
    foreach ($snapshots ?: [] as $snapshot) {
      if ($snapshot->getName() !== $snapshot->getSnapshotId()) {
        $options[$snapshot->getSnapshotId()] = "{$snapshot->getName()} ({$snapshot->getSnapshotId()})";
      }
      else {
        $options[$snapshot->getSnapshotId()] = $snapshot->getSnapshotId();
      }
    }

    natcasesort($options);
    return $options;
  }

}
